#include "p24fj48ga002.h"



/*************************************
**functions prototypes				**
void InitI2C(unsigned int FSCL);
uns8 read_register(uns8 register_name);
void write_register(uns8 register_name, uns8 register_value);
void i2c_ack_polling(uns8 device_address);
void i2c_start(void);
void i2c_stop(void);
uns8 i2c_read_byte(void);
bit i2c_send_byte(uns8 out_byte);

*************************************/
//The Slave ADdress (SAD) associated to the LIS302DL is 0011100 (when SDO is tied to GND), 
//lsb indicates the R/W mode.
#define DEVICE_WRITE    0x38 //Default LIS302DL I2C address - write
#define DEVICE_READ     0x39 //Default LIS302DL I2C address - read

#define ACK     1
#define NO_ACK  0

//#define I2C_DELAY   10

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!MODIFICAR PARA O PIC EM QUESTAO
//#define WRITE_sda() TRISB = TRISB & 0b.1011.1111 //SDA must be output when writing
//#define READ_sda()  TRISB = TRISB | 0b.0100.0000 //SDA must be input when reading - don't forget the resistor on SDA!!

void InitI2C(unsigned int FSCL)
{
/*	I2C1CONbits. //This register (Register24-1) allows control of the module's operation. 
	
	I2C1STAT //This register (Register24-2) contains status flags indicating the module's state during operation.
	
	I2C1MSK	//This register (Register24-3) designates which bit positions 
			//in I2C1ADD can be ignored, which allows for multiple address support. 
*/	
	FCY = Fosc/2;
	//To compute the Baud Rate Generator reload value, use the following equation:
	I2C1BRG = (FCY/FSCL) - (FCY/10000000) - 1;
		
	I2C1CONbits.A10M = 0; //I2CxADD is a 7-bit slave address 
	
	I2C1CONbits.DISSLW = 0;//slew rate control of the device pin output

	I2C1CONbits.I2CEN = 1; //ENABLING I2C OPERATION


	/* ININCIO DA COMUNICACAO COM O ACELEROMETRO*/
	uns8 response;

    response = read_register(0x0F); //Who am I
    printf("\n\rWho am I (should be 0x3B): %h\n\r", response);

    write_register(0x20, 0b.0100.0111); //Power on device, enable all axis, and turn off self test - CTRL_REG1

}


uns8 read_register(uns8 register_name)
{
    uns8 in_byte;
    
    i2c_ack_polling(DEVICE_WRITE);

    i2c_start();
    i2c_send_byte(DEVICE_WRITE); 
    i2c_send_byte(register_name); //Write register address   
    //i2c_stop();

    i2c_start(); //Repeat start (SR)
    i2c_send_byte(DEVICE_READ); //Now ask the IC to report on the last command
    in_byte = i2c_read_byte();
    i2c_stop();
        
    return(in_byte);
}

void write_register(uns8 register_name, uns8 register_value)
{
    i2c_ack_polling(DEVICE_WRITE);

    i2c_start();
    i2c_send_byte(DEVICE_WRITE); 
    i2c_send_byte(register_name); //Write register address
    i2c_send_byte(register_value); //Write data
    i2c_stop();

    //Return nothing
}

/*************************************
**Software I2C routines				**
*************************************/
void i2c_ack_polling(uns8 device_address)
{
    while(1)
    {
        i2c_start();
        if (i2c_send_byte(device_address) == ACK) break;
    }
    i2c_stop();
}

void i2c_start(void)
{
	//PIC 24F code
	//ensure that the bus is in an Idle state
	i(f (I2C1STATbits.P == 1) // Indicates that a Stop bit has been detected last
	{
		I2C1CONbits.SEN = 1; //sets the Start Enable bit
	}
	//
    

	WRITE_sda();
    SDA = 1;
    delay_us(I2C_DELAY);

    SCL = 1;
    delay_us(I2C_DELAY);

    SDA = 0;
    delay_us(I2C_DELAY);
}

//The I2C Clock has a minimum of 2us high time and 2us low time
void i2c_stop(void)
{
    SCL = 0;
    delay_us(I2C_DELAY);

    WRITE_sda();

    SDA = 0;
    delay_us(I2C_DELAY);

    SCL = 1;
    delay_us(I2C_DELAY);

    SDA = 1;
    delay_us(I2C_DELAY);
}

//The I2C Clock has a minimum of 2us high time and 2us low time
//8MHz = 0.5us per instruction
uns8 i2c_read_byte(void)
{
    int j, in_byte;

    SCL = 0;

    READ_sda();

    for(j = 0 ; j < 8 ; j++)
    {
        SCL = 0;
        delay_us(I2C_DELAY);

        SCL = 1;
        delay_us(I2C_DELAY);

        in_byte = rl(in_byte);
        in_byte.0 = SDA;
    }

    //Send 9th bit acknowledge
/*
    For the LIS, single byte reads, there is NMAK or no master acknowledge
    SCL = 0;
    WRITE_sda();
    SDA = 0;
    delay_us(I2C_DELAY);
    SCL = 1;

    delay_us(I2C_DELAY);
    SCL = 0;

    delay_us(I2C_DELAY);
    SDA = 1;
    while(1);
*/
    return(in_byte);
}

//The I2C Clock has a minimum of 2us high time and 2us low time
//8MHz = 500ns per instruction
bit i2c_send_byte(uns8 out_byte)
{
    uns8 i;

    WRITE_sda();

    for(i = 0 ; i < 8 ; i++)
    {
        SCL = 0;
        delay_us(I2C_DELAY);

        out_byte = rl(out_byte);
        SDA = Carry;
        delay_us(5);

        SCL = 1;
        delay_us(I2C_DELAY);
    }

    //Read ack
    SCL = 0;
    delay_us(I2C_DELAY);

    READ_sda();

    SCL = 1;
    delay_us(I2C_DELAY);

    //Wait for IC to acknowledge
    for(i = 0 ; i < 255 ; i++)
        if(SDA == 0) break;

    SCL = 0;
    delay_us(I2C_DELAY);

    if (i == 255) return(NO_ACK);
    
    return(ACK);
}

/*************************************
**Auxiliary I2C routines			**
*************************************/
//General short delay
void delay_ms(uns16 x)
{
    //Clocks out at 1006us per 1ms
    uns8 y, z;
    for ( ; x > 0 ; x--)
        for ( y = 0 ; y < 4 ; y++)
            for ( z = 0 ; z < 69 ; z++);
}

//Really short delay
void delay_us(uns16 x)
{
    //Calling with 1us returns 7.5us
    //Calling with 10us returns 48
    //Calling with 1000us returns 4.5ms
    while(--x); 
}